﻿/*

  In URL field you can type internet address or path to local html file.

  Keyboard:
    Alt+Enter - maximize/restore main window
    Esc       - quit
    in grid:
      Enter        - view image (also double click on item of image list)
      Ctrl+C       - copy image address to clipboard
      Ctrl+Shift+C - copy all image addresses to clipboard
      Ctrl+S       - save as
      Ctrl+W       - close view

*/

#include "hmg.ch"

MEMVAR _HMG_SYSDATA

STATIC slKeepRatio := .T.
STATIC snPosView   := 0
STATIC scTmpDir


FUNCTION Main()
  LOCAL aImg := { {"Eiffel Tower", "http://media.thisisinsider.com/images/58d919eaf2d0331b008b4bbd-960-720.jpg"}, ;
                  {"London Tower", "https://i2.wp.com/travelbluebook.com/wp-content/uploads/2016/02/Tower-of-London.jpg"}, ;
                  {"Pisa Tower",   "http://media1.britannica.com/eb-media/88/80588-004-0B5DCB41.jpg"}, ;
                  {"Taj Mahal",    "http://images.says.com/uploads/story_source/source_image/547108/2b63.jpg"}, ;
                  {"Tower Bridge", "http://www.goandco.co.uk/images/offices/tower-bridge.jpg"} }
  LOCAL n

  SET FONT TO "MS Shell Dlg", 8

  DEFINE WINDOW MainWnd;
    WIDTH  480 + GetSystemMetrics(7 /*SM_CXFIXEDFRAME*/) * 2;
    HEIGHT 320 + GetSystemMetrics(4 /*SM_CYCAPTION*/) + GetSystemMetrics(8 /*SM_CYFIXEDFRAME*/) * 2;
    TITLE  "Image from website";
    MAIN;
    ON PAINT    PaintFrame(MainWnd.HANDLE, 41, 170, MainWnd.CLIENTAREAWIDTH - 180, MainWnd.CLIENTAREAHEIGHT - 51 - GetWindowHeight(MainWnd.STATUSBAR.HANDLE));
    ON SIZE     MainWndResize();
    ON MAXIMIZE MainWndResize();
    ON RELEASE  HB_DirRemoveAll(scTmpDir)

    DEFINE LABEL UrlLabel
      ROW    13
      COL    10
      WIDTH  24
      HEIGHT 13
      VALUE  "URL:"
    END LABEL

    DEFINE TEXTBOX UrlTBox
      ROW      10
      COL      35
      HEIGHT   21
      DATATYPE CHARACTER
      ONCHANGE (MainWnd.UrlButton.ENABLED := (! Empty(This.VALUE)))
    END TEXTBOX

    DEFINE BUTTON UrlButton
      ROW     9
      WIDTH   20
      HEIGHT  23
      CAPTION "->"
      ACTION  ImageFind(aImg)
    END BUTTON

    DEFINE GRID PicGrid
      ROW            41
      COL            10
      WIDTH          150
      SHOWHEADERS    .F.
      WIDTHS         {0}
      CELLNAVIGATION .F.
      ONCHANGE       UpdateStatus(aImg)
      ONDBLCLICK     ImageView(aImg)
      ONKEY          PicGridOnKey(aImg)
      ONGOTFOCUS     PostMessage(MainWnd.ViewButton.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS    SendMessage(MainWnd.ViewButton.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
    END GRID

    DEFINE CHECKBOX RatioChBox
      COL         10
      WIDTH       150
      HEIGHT      16
      VALUE       slKeepRatio
      CAPTION     "&Keep original aspect ratio"
      ONCHANGE    ((slKeepRatio := This.VALUE), MainWndResize())
    END CHECKBOX

    DEFINE BUTTON ViewButton
      COL     10
      WIDTH   70
      HEIGHT  23
      CAPTION "View"
      ACTION  ImageView(aImg)
    END BUTTON

    DEFINE BUTTON SaveButton
      COL     90
      WIDTH   70
      HEIGHT  23
      CAPTION "Save as"
      ACTION  ImageSaveAs(aImg)
    END BUTTON

    DEFINE IMAGE PicImage
      STRETCH .T.
    END IMAGE

    DEFINE STATUSBAR
      STATUSITEM ""
      STATUSITEM "" WIDTH 90
    END STATUSBAR

    ON KEY ALT+RETURN ACTION If(MainWnd.IsMaximized, MainWnd.RESTORE, MainWnd.MAXIMIZE)
    ON KEY ESCAPE     ACTION MainWnd.RELEASE
  END WINDOW

  FOR n := 1 TO Len(aImg)
    MainWnd.PicGrid.AddItem({aImg[n][1]})
  NEXT

  MainWnd.UrlButton.ENABLED := .F.
  MainWnd.PicGrid.VALUE     := 1

  EventCreate({ || SetMinMaxTrackSize(EventLPARAM(), 300, 200) }, MainWnd.HANDLE, 36 /*WM_GETMINMAXINFO*/)

  MainWndResize()
  MainWnd.CENTER
  MainWnd.ACTIVATE

RETURN NIL


FUNCTION MainWndResize()
  LOCAL nHBmp := MainWnd.PicImage.HBITMAP
  LOCAL nCAW  := MainWnd.CLIENTAREAWIDTH
  LOCAL nCAH  := MainWnd.CLIENTAREAHEIGHT - GetWindowHeight(MainWnd.STATUSBAR.HANDLE)
  LOCAL nPicR := 42
  LOCAL nPicC := 171
  LOCAL nPicW := nCAW - 182
  LOCAL nPicH := nCAH - 53
  LOCAL nPicRatio, nBmpRatio

  IF slKeepRatio .and. (nHBmp != 0)
    nPicRatio := nPicW / nPicH
    nBmpRatio := BT_BitmapWidth(nHBmp) / BT_BitmapHeight(nHBmp)

    IF nPicRatio < nBmpRatio
      nPicH := Int(nPicW / nBmpRatio)
      nPicR += Int((nCAH - 53 - nPicH) / 2)
    ELSEIF nPicRatio > nBmpRatio
      nPicW := Int(nPicH * nBmpRatio)
      nPicC += Int((nCAW - 182 - nPicW) / 2)
    ENDIF
  ENDIF

  MainWnd.UrlTBox.WIDTH   := nCAW - 65
  MainWnd.UrlButton.COL   := nCAW - 30
  MainWnd.PicGrid.HEIGHT  := nCAH - 110
  MainWnd.RatioChBox.ROW  := nCAH - 59
  MainWnd.ViewButton.ROW  := nCAH - 33
  MainWnd.SaveButton.ROW  := nCAH - 33
  MainWnd.PicImage.ROW    := nPicR
  MainWnd.PicImage.COL    := nPicC
  MainWnd.PicImage.WIDTH  := nPicW
  MainWnd.PicImage.HEIGHT := nPicH

  MainWnd.PicGrid.ColumnWIDTH(1) := 150 - 4 - If(MainWnd.PicGrid.ITEMCOUNT > ListViewGetCountPerPage(MainWnd.PicGrid.HANDLE), GetVScrollBarWidth(), 0)

  MainWnd.PicImage.REDRAW

RETURN NIL


FUNCTION UpdateStatus(aImg)

  IF MainWnd.PicGrid.ITEMCOUNT == 0
    MainWnd.STATUSBAR.Item(1) := ""
    MainWnd.STATUSBAR.Item(2) := ""
  ELSE
    MainWnd.STATUSBAR.Item(1) := aImg[MainWnd.PicGrid.VALUE][2]
    MainWnd.STATUSBAR.Item(2) := If(MainWnd.PicGrid.VALUE == snPosView, HB_NtoS(BT_BitmapWidth(MainWnd.PicImage.HBITMAP)) + "x" + HB_NtoS(BT_BitmapHeight(MainWnd.PicImage.HBITMAP)), "")
  ENDIF

RETURN NIL


FUNCTION PicGridOnKey(aImg)

  SWITCH HMG_GetLastVirtualKeyDown()
    CASE VK_C
    CASE VK_INSERT
      HMG_CleanLastVirtualKeyDown()
      IF (GetKeyState(VK_CONTROL) < 0) .and. (GetKeyState(VK_MENU) >= 0)
        ImageCopyUrl(aImg, (GetKeyState(VK_SHIFT) < 0))
      ENDIF
      EXIT
    CASE VK_S
      HMG_CleanLastVirtualKeyDown()
      IF (GetKeyState(VK_CONTROL) < 0) .and. (GetKeyState(VK_SHIFT) >= 0) .and. (GetKeyState(VK_MENU) >= 0)
        ImageSaveAs(aImg)
      ENDIF
      EXIT
    CASE VK_W
      HMG_CleanLastVirtualKeyDown()
      IF (GetKeyState(VK_CONTROL) < 0) .and. (GetKeyState(VK_SHIFT) >= 0) .and. (GetKeyState(VK_MENU) >= 0)
        ImageClose(aImg)
      ENDIF
      EXIT
  ENDSWITCH

RETURN NIL


FUNCTION ImageFind(aImg)
  LOCAL cUrl    := AllTrim(MainWnd.UrlTBox.VALUE)
  LOCAL cProto1 := "http://"
  LOCAL cProto2 := "https://"
  LOCAL cHtmlFile
  LOCAL cHtml
  LOCAL aUrl
  LOCAL cImgUrl
  LOCAL n, n1, n2, n3, n4

  IF Empty(scTmpDir) .or. (! HB_DirExists(scTmpDir))
    scTmpDir := DirTmpCreate()
  ENDIF

  IF (HB_UTF8Left(cUrl, 7) == cProto1) .or. (HB_UTF8Left(cUrl, 8) == cProto2)
    cHtmlFile := scTmpDir + "s.htm"

    IF ! URLDownloadToFile(cUrl, cHtmlFile)
      MsgBox("Can not download:" + CRLF + cUrl, MainWnd.TITLE)
      RETURN NIL
    ENDIF
  ELSE
    cHtmlFile := cUrl

    IF ! HB_FileExists(cHtmlFile)
      MsgBox("File does not exist:" + CRLF + cHtmlFile, MainWnd.TITLE)
      RETURN NIL
    ENDIF
  ENDIF

  cHtml := HB_MemoRead(cHtmlFile)
  aUrl  := {}
  n1    := 1

  DO WHILE (n1 := HB_UTF8At("<img", cHtml, n1)) > 0
    IF (n2 := HB_UTF8At(">", cHtml, n1 + 4)) == 0
      EXIT
    ENDIF

    IF (n3 := HB_UTF8At(' src="', cHtml, n1 + 4, n2 - 1)) > 0
      n3 += 6
      n4 := HB_UTF8At('"', cHtml, n3, n2 - 1)
    ELSE
      n4 := 0
    ENDIF

    IF (n3 == 0) .or. (n4 == 0)
      IF (n3 := HB_UTF8At(' data-src="', cHtml, n1 + 4, n2 - 1)) > 0
        n3 += 11

        IF (n4 := HB_UTF8At('"', cHtml, n3, n2 - 1)) == 0
          n3 := n4
        ENDIF
      ELSE
        n4 := 0
      ENDIF
    ENDIF

    cImgUrl := HB_UTF8SubStr(cHtml, n3, n4 - n3)

    IF ! Empty(cImgUrl)
      IF (! (HB_UTF8Left(cImgUrl, 7)) == cProto1) .and. (! (HB_UTF8Left(cImgUrl, 8) == cProto2))
        IF HB_UTF8Left(cImgUrl, 1) == "/"
          cImgUrl := HB_UTF8Left(cUrl, HB_UTF8At("/", cUrl, 9) - 1) + cImgUrl
        ELSEIF HB_UTF8Left(cImgUrl, 2) == "./"
          cImgUrl := HB_UTF8Left(cUrl, HB_UTF8RAt("/", cUrl)) + HB_UTF8SubStr(cImgUrl, 3)
        ELSEIF HB_UTF8Left(cImgUrl, 3) == "../"
          n := HB_UTF8RAt("/", cUrl)

          DO WHILE HB_UTF8Left(cImgUrl, 3) == "../"
            cImgUrl := HB_UTF8SubStr(cImgUrl, 4)
            n := HB_UTF8RAt("/", cUrl, 0, n - 1)
          ENDDO

          cImgUrl := HB_UTF8Left(cUrl, n) + cImgUrl
        ELSE
          cImgUrl := HB_UTF8Left(cUrl, HB_UTF8RAt("/", cUrl)) + cImgUrl
        ENDIF
      ENDIF

      aAdd(aUrl, cImgUrl)
    ENDIF

    n1 := n2 + 1
  ENDDO

  IF Empty(aUrl)
    MsgBox("Not found images at:" + CRLF + cUrl, MainWnd.TITLE)
  ELSE
    ImageClose(aImg)
    HB_FileDelete(scTmpDir + "*.*", "HRS")
    aSize(aImg, 0)
    MainWnd.PicGrid.DELETEALLITEMS

    FOR n := 1 TO Len(aUrl)
      aAdd(aImg, {HB_NtoS(n), aUrl[n]})
      MainWnd.PicGrid.AddItem({aImg[n][1]})
    NEXT

    MainWnd.PicGrid.VALUE := 1
    MainWndResize()
  ENDIF

RETURN NIL


FUNCTION ImageCopyUrl(aImg, lAll)
  LOCAL cText
  LOCAL n

  MainWnd.PicGrid.VALUE := MainWnd.PicGrid.CELLROWFOCUSED

  IF lAll
    cText := ""

    FOR n := 1 TO Len(aImg)
      cText += aImg[n][2] + CRLF
    NEXT

    SetClipboard(HB_StrShrink(cText, 2))
  ELSE
    SetClipboard(aImg[MainWnd.PicGrid.VALUE][2])
  ENDIF

RETURN NIL


FUNCTION ImageView(aImg)
  LOCAL nPos := MainWnd.PicGrid.VALUE
  LOCAL cTmpFile

  IF nPos != MainWnd.PicGrid.CELLROWFOCUSED
    MainWnd.PicGrid.VALUE := MainWnd.PicGrid.CELLROWFOCUSED
    RETURN NIL
  ENDIF

  cTmpFile := ImageDownload(aImg, nPos)

  IF ! Empty(cTmpFile)
    IF snPosView > 0
      MainWnd.PicGrid.CellEx(snPosView, 1) := aImg[snPosView][1]
    ENDIF

    snPosView := nPos
    MainWnd.PicGrid.CellEx(nPos, 1) := "  " + aImg[nPos][1]
    MainWnd.PicImage.PICTURE := cTmpFile

    MainWndResize()
    UpdateStatus(aImg)
  ENDIF

RETURN NIL


FUNCTION ImageClose(aImg)

  MainWnd.PicGrid.VALUE := MainWnd.PicGrid.CELLROWFOCUSED

  IF snPosView > 0
    MainWnd.PicGrid.CellEx(snPosView, 1) := aImg[snPosView][1]
    MainWnd.PicImage.PICTURE := ""
    snPosView := 0

    MainWndResize()
    UpdateStatus(aImg)
  ENDIF

RETURN NIL


FUNCTION ImageSaveAs(aImg)
  LOCAL nPos
  LOCAL cExt
  LOCAL cTargetFile
  LOCAL cTmpFile

  MainWnd.PicGrid.VALUE := MainWnd.PicGrid.CELLROWFOCUSED

  nPos        := MainWnd.PicGrid.VALUE
  cExt        := GetExtension(aImg[nPos][2])
  cTargetFile := PutFile({{"*" + cExt, "*" + cExt}}, NIL, HB_CWD(), .F., aImg[nPos][1] + cExt, cExt)

  IF ! Empty(cTargetFile)
    cTmpFile := ImageDownload(aImg, nPos)

    IF ! Empty(cTmpFile)
      IF HB_fCopy(cTmpFile, cTargetFile) != 0
        MsgBox("Can not save file:" + CRLF + cTargetFile, MainWnd.TITLE)
      ENDIF
    ENDIF
  ENDIF

RETURN NIL


FUNCTION ImageDownload(aImg, nPos)
  LOCAL cTmpFile

  IF Empty(scTmpDir) .or. (! HB_DirExists(scTmpDir))
    scTmpDir := DirTmpCreate()
  ENDIF

  cTmpFile := scTmpDir + HB_NtoS(nPos) + GetExtension(aImg[nPos][2])

  IF ! HB_FileExists(cTmpFile)
    IF ! URLDownloadToFile(aImg[nPos][2], cTmpFile)
      cTmpFile := ""
      MsgBox("Can not download image:" + CRLF + aImg[nPos][2], MainWnd.TITLE)
    ENDIF
  ENDIF

RETURN cTmpFile


FUNCTION GetExtension(cName)
  LOCAL cExt := Lower(HB_TokenGet(cName, HB_TokenCount(cName, "."), "."))

  IF ! ((cExt == "jpeg") .or. (cExt == "jpg") .or. (cExt == "png") .or. (cExt == "gif") .or. (cExt == "bmp"))
    cExt := "jpg"
  ENDIF

RETURN "." + cExt


FUNCTION DirTmpCreate()
  LOCAL cTmpDir := GetTempDir()
  LOCAL cName   := HB_fNameName(GetProgramFileName())
  LOCAL nCount  := 0

  HB_DirBuild(cTmpDir)

  DO WHILE HB_fNameExists(cTmpDir + cName + "_" + HB_NtoS(nCount))
    ++nCount
  ENDDO

  HB_DirCreate(cTmpDir += cName + "_" + HB_NtoS(nCount) + "\")

RETURN cTmpDir


#pragma BEGINDUMP

#include "SET_COMPILE_HMG_UNICODE.ch"
#include "HMG_UNICODE.h"

#include <Windows.h>
#include <urlmon.h>
#include "hbapi.h"


      // SetMinMaxTrackSize(lParam, nMinX, nMinY, nMaxX, nMaxY)
HB_FUNC( SETMINMAXTRACKSIZE )
{
  MINMAXINFO *MinMax = (MINMAXINFO *) HMG_parnl(1);

  if (hb_parni(2) > 0)
    MinMax->ptMinTrackSize.x = hb_parni(2);
  if (hb_parni(3) > 0)
    MinMax->ptMinTrackSize.y = hb_parni(3);
  if (hb_parni(4) > 0)
    MinMax->ptMaxTrackSize.x = hb_parni(4);
  if (hb_parni(5) > 0)
    MinMax->ptMaxTrackSize.y = hb_parni(5);
}


      // PaintFrame(nHWnd, nRow, nCol, nWidth, nHeight)
HB_FUNC( PAINTFRAME )
{
  HWND        hWnd;
  PAINTSTRUCT ps;
  RECT        rc;
  HDC         hdc;

  hWnd = (HWND) HMG_parnl(1);
  hdc  = BeginPaint(hWnd, &ps);

  if (hdc)
  {
    rc.left   = (LONG) hb_parnl(3);
    rc.top    = (LONG) hb_parnl(2);
    rc.right  = rc.left + (LONG) hb_parnl(4);
    rc.bottom = rc.top  + (LONG) hb_parnl(5);

    DrawEdge(hdc, &rc, EDGE_BUMP, BF_FLAT|BF_RECT);

    EndPaint(hWnd, &ps);
  }
}


      // https://msdn.microsoft.com/en-us/library/ms775123(v=vs.85).aspx
      // URLDownloadToFile(cURL, cFile)
HB_FUNC( URLDOWNLOADTOFILE )
{
  HRESULT hr = URLDownloadToFile(NULL, HMG_parc(1), HMG_parc(2), 0, NULL);
  
  hb_retl(hr == S_OK);
}

#pragma ENDDUMP
